@@ -35,7 +35,10 @@ use crate::client::rpc::{
     TSInsertStringRecordReq, TSInsertTabletReq, TSOpenSessionReq, TSProtocolVersion,
     TTSIServiceSyncClient,
 };
-use crate::protocal::{TSDataType, FLAG, MULTIPLE_ERROR, NEED_REDIRECTION, SUCCESS_STATUS};
+use crate::protocal::{
+    TSCompressionType, TSDataType, TSEncoding, FLAG, MULTIPLE_ERROR, NEED_REDIRECTION,
+    SUCCESS_STATUS,
+};
 
 use super::rpc::{
     TSDeleteDataReq, TSExecuteStatementReq, TSInsertRecordReq, TSInsertRecordsOfOneDeviceReq,
@@ -45,7 +48,9 @@ use super::{
     rpc::{TSCloseSessionReq, TSStatus},
     RowRecord,
 };
-use super::{DataSet, Dictionary, Session, Value};
+use super::{DataSet, Dictionary, Result, Session, Value};
+
+static DEFAULT_TIME_ZONE: &str = "Asia/Shanghai";
 
 #[derive(Debug, Clone)]
 
@@ -71,7 +76,7 @@ impl Default for Config {
             password: String::from("root"),
             timeout_ms: Some(30000),
             fetch_size: 1000,
-            timezone: Some(String::from("Asia/Shanghai")),
+            timezone: Some(String::from(DEFAULT_TIME_ZONE)),
             enable_compression: false,
             protocol_version: TSProtocolVersion::IOTDB_SERVICE_PROTOCOL_V3,
             is_align: true,
@@ -87,41 +92,41 @@ pub struct RpcSession {
 }
 
 impl<'a> RpcSession {
-    pub fn new(config: &Config) -> Result<Self, Box<dyn Error>> {
+    pub fn new(config: &Config) -> Result<Self> {
         let mut tcp_channel = TTcpChannel::new();
         let endpint = format!("{}:{}", config.host, config.port);
-        match tcp_channel.open(&endpint) {
-            Ok(_) => {
-                let (i_chan, o_chan) = tcp_channel.split()?;
 
-                let (i_prot, o_prot) = (
-                    TFramedReadTransport::new(i_chan),
-                    TFramedWriteTransport::new(o_chan),
-                );
+        tcp_channel.open(&endpint).map_err(|err| {
+            Box::<dyn Error>::from(
+                format!("failed to connect to {}, {:?}", endpint, err).to_string(),
+            )
+        })?;
 
-                let (input_protocol, output_protocol): (
-                    Box<dyn TInputProtocol>,
-                    Box<dyn TOutputProtocol>,
-                ) = match config.enable_compression {
-                    false => (
-                        Box::new(TBinaryInputProtocol::new(i_prot, true)),
-                        Box::new(TBinaryOutputProtocol::new(o_prot, true)),
-                    ),
-                    true => (
-                        Box::new(TCompactInputProtocol::new(i_prot)),
-                        Box::new(TCompactOutputProtocol::new(o_prot)),
-                    ),
-                };
+        let (i_chan, o_chan) = tcp_channel.split()?;
 
-                Ok(Self {
-                    config: config.clone(),
-                    session_id: None,
-                    statement_id: -1,
-                    client: TSIServiceSyncClient::new(input_protocol, output_protocol),
-                })
-            }
-            Err(err) => Err(format!("failed to connect to {}, {:?}", endpint, err).into()),
-        }
+        let (i_prot, o_prot) = (
+            TFramedReadTransport::new(i_chan),
+            TFramedWriteTransport::new(o_chan),
+        );
+
+        let (input_protocol, output_protocol): (Box<dyn TInputProtocol>, Box<dyn TOutputProtocol>) =
+            match config.enable_compression {
+                false => (
+                    Box::new(TBinaryInputProtocol::new(i_prot, true)),
+                    Box::new(TBinaryOutputProtocol::new(o_prot, true)),
+                ),
+                true => (
+                    Box::new(TCompactInputProtocol::new(i_prot)),
+                    Box::new(TCompactOutputProtocol::new(o_prot)),
+                ),
+            };
+
+        Ok(Self {
+            config: config.clone(),
+            session_id: None,
+            statement_id: -1,
+            client: TSIServiceSyncClient::new(input_protocol, output_protocol),
+        })
     }
 }
 
@@ -194,7 +199,11 @@ impl<'a> Iterator for RpcDataSet<'a> {
 
             self.row_index += 1;
 
-            let mut output_values: Vec<Value> = Vec::with_capacity(self.get_column_names().len());
+            let mut output_values: Vec<Value> = Vec::with_capacity(if self.is_ignore_timestamp() {
+                self.get_column_names().len()
+            } else {
+                self.get_column_names().len() + 1
+            });
             if !self.is_ignore_timestamp() {
                 output_values.push(Value::Int64(self.timestamp));
             }
@@ -229,6 +238,7 @@ pub struct RpcDataSet<'a> {
 }
 
 impl<'a> RpcDataSet<'a> {
+    #[allow(dead_code)]
     fn is_null(&self, column_index: usize, row_index: usize) -> bool {
         let bitmap = self.bitmaps[column_index];
         let shift = row_index % 8;
@@ -240,68 +250,83 @@ impl<'a> RpcDataSet<'a> {
             return false;
         }
         if self.query_data_set.time.len() == 0 {
-            //Fetching result from iotdb server
-            match self
-                .session
-                .client
-                .fetch_results(super::rpc::TSFetchResultsReq {
-                    session_id: self.session.session_id.unwrap(),
-                    statement: self.statement.clone(),
-                    fetch_size: self.session.config.fetch_size,
-                    query_id: self.query_id,
-                    is_align: self.session.config.is_align,
-                    timeout: self.session.config.timeout_ms,
-                }) {
-                Ok(resp) => {
-                    let status = resp.status;
-                    match check_status(status) {
-                        Ok(_) => {
-                            if resp.has_result_set {
-                                //update query_data_set and release row_index
-                                self.query_data_set = resp.query_data_set.unwrap();
-                                self.row_index = 0;
-                            } else {
-                                //Auto close the dataset when no result.
-                                match self.session.client.close_operation(
-                                    super::rpc::TSCloseOperationReq {
-                                        session_id: self.session.session_id.unwrap(),
-                                        query_id: Some(self.query_id),
-                                        statement_id: Some(self.session.statement_id),
-                                    },
-                                ) {
-                                    Ok(status) => match check_status(status) {
-                                        Ok(_) => {
-                                            self.closed = true;
-                                            return false;
-                                        }
-                                        Err(err) => {
-                                            eprint!(
-                                                "An error occurred when closing dataset {:?}",
-                                                err
-                                            )
-                                        }
-                                    },
-                                    Err(err) => {
-                                        eprint!("An error occurred when closing dataset {:?}", err)
+            if let Some(session_id) = self.session.session_id {
+                //Fetching result from iotdb server
+                match self
+                    .session
+                    .client
+                    .fetch_results(super::rpc::TSFetchResultsReq {
+                        session_id: session_id,
+                        statement: self.statement.clone(),
+                        fetch_size: self.session.config.fetch_size,
+                        query_id: self.query_id,
+                        is_align: self.session.config.is_align,
+                        timeout: self.session.config.timeout_ms,
+                    }) {
+                    Ok(resp) => {
+                        let status = resp.status;
+                        match check_status(status) {
+                            Ok(_) => {
+                                if resp.has_result_set {
+                                    //update query_data_set and release row_index
+                                    if let Some(query_data_set) = resp.query_data_set {
+                                        self.query_data_set = query_data_set;
+                                        self.row_index = 0;
                                     }
+                                } else {
+                                    //Auto close the dataset when it doesn't have any results on the server.
+                                    self.close();
+                                    return false;
                                 }
                             }
-                        }
-                        Err(err) => {
-                            eprint!("An error occurred when fetch result: {}", err);
-                            return false;
+                            Err(err) => {
+                                eprint!("An error occurred when fetch result: {}", err);
+                                return false;
+                            }
                         }
                     }
-                }
-                Err(err) => {
-                    eprint!("An error occurred when fetch result: {}", err);
-                    return false;
+                    Err(err) => {
+                        eprint!("An error occurred when fetch result: {}", err);
+                        return false;
+                    }
                 }
             }
         }
-
         self.query_data_set.time.len() > 0
     }
+
+    pub fn close(&mut self) {
+        if !self.closed {
+            if let Some(session_id) = self.session.session_id {
+                match self
+                    .session
+                    .client
+                    .close_operation(super::rpc::TSCloseOperationReq {
+                        session_id: session_id,
+                        query_id: Some(self.query_id),
+                        statement_id: Some(self.session.statement_id),
+                    }) {
+                    Ok(status) => match check_status(status) {
+                        Ok(_) => {
+                            self.closed = true;
+                        }
+                        Err(err) => {
+                            eprint!("An error occurred when closing dataset {:?}", err)
+                        }
+                    },
+                    Err(err) => {
+                        eprint!("An error occurred when closing dataset {:?}", err)
+                    }
+                }
+            }
+        }
+    }
+}
+
+impl<'a> Drop for RpcDataSet<'a> {
+    fn drop(&mut self) {
+        self.close();
+    }
 }
 
 impl<'a> DataSet for RpcDataSet<'a> {
@@ -320,7 +345,7 @@ impl<'a> DataSet for RpcDataSet<'a> {
         if self.is_ignore_timestamp() {
             self.data_types.clone()
         } else {
-            //Include the time column data type
+            //Include the time column
             let mut column_types = vec![TSDataType::Int64];
             column_types.extend(self.data_types.clone());
             column_types
@@ -336,7 +361,7 @@ impl<'a> DataSet for RpcDataSet<'a> {
     }
 }
 
-fn check_status(status: TSStatus) -> Result<(), Box<dyn Error>> {
+fn check_status(status: TSStatus) -> Result<()> {
     match status.code {
         SUCCESS_STATUS | NEED_REDIRECTION => Ok(()),
         MULTIPLE_ERROR => {
@@ -367,18 +392,18 @@ fn check_status(status: TSStatus) -> Result<(), Box<dyn Error>> {
     }
 }
 
-fn fire_closed_error() -> Result<(), Box<dyn Error>> {
+fn fire_closed_error() -> Result<()> {
     Err("Operation can't be performed, the session is closed.".into())
 }
 
 impl<'a> Session<'a> for RpcSession {
-    fn open(&mut self) -> Result<(), Box<dyn Error>> {
+    fn open(&mut self) -> Result<()> {
         let resp = self.client.open_session(TSOpenSessionReq::new(
             self.config.protocol_version,
             self.config
                 .timezone
                 .clone()
-                .unwrap_or(Config::default().timezone.unwrap()),
+                .unwrap_or(DEFAULT_TIME_ZONE.to_string()),
             self.config.username.clone(),
             self.config.password.clone(),
             None,
@@ -393,7 +418,7 @@ impl<'a> Session<'a> for RpcSession {
         }
     }
 
-    fn close(&mut self) -> Result<(), Box<dyn Error>> {
+    fn close(&mut self) -> Result<()> {
         if let Some(session_id) = self.session_id {
             let status = self
                 .client
@@ -405,29 +430,26 @@ impl<'a> Session<'a> for RpcSession {
         }
     }
 
-    fn set_storage_group(&mut self, storage_group_id: &str) -> Result<(), Box<dyn Error>> {
+    fn set_storage_group(&mut self, storage_group_id: &str) -> Result<()> {
         if let Some(session_id) = self.session_id {
             let status = self
                 .client
-                .set_storage_group(session_id, storage_group_id.to_string())?;
+                .set_storage_group(session_id, storage_group_id.into())?;
             check_status(status)
         } else {
             fire_closed_error()
         }
     }
 
-    fn delete_storage_group(&mut self, storage_group_id: &str) -> Result<(), Box<dyn Error>> {
+    fn delete_storage_group(&mut self, storage_group_id: &str) -> Result<()> {
         self.delete_storage_groups(vec![storage_group_id])
     }
 
-    fn delete_storage_groups(
-        &mut self,
-        storage_group_ids: Vec<&str>,
-    ) -> Result<(), Box<dyn Error>> {
+    fn delete_storage_groups(&mut self, storage_group_ids: Vec<&str>) -> Result<()> {
         if let Some(session_id) = self.session_id {
             let status = self.client.delete_storage_groups(
                 session_id,
-                storage_group_ids.iter().map(|x| x.to_string()).collect(),
+                storage_group_ids.iter().map(ToString::to_string).collect(),
             )?;
             check_status(status)
         } else {
@@ -445,7 +467,7 @@ impl<'a> Session<'a> for RpcSession {
         attributes: T,
         tags: T,
         measurement_alias: Option<String>,
-    ) -> Result<(), Box<dyn Error>>
+    ) -> Result<()>
     where
         T: Into<Option<Dictionary>>,
     {
@@ -477,7 +499,7 @@ impl<'a> Session<'a> for RpcSession {
         attributes_list: T,
         tags_list: T,
         measurement_alias_list: Option<Vec<String>>,
-    ) -> Result<(), Box<dyn Error>>
+    ) -> Result<()>
     where
         T: Into<Option<Vec<Dictionary>>>,
     {
@@ -486,27 +508,12 @@ impl<'a> Session<'a> for RpcSession {
                 .client
                 .create_multi_timeseries(TSCreateMultiTimeseriesReq::new(
                     session_id,
-                    paths.iter().map(|x| x.to_string()).collect(),
-                    data_types
-                        .iter()
-                        .map(|t| {
-                            let n: i32 = t.into();
-                            n
-                        })
-                        .collect(),
-                    encodings
-                        .iter()
-                        .map(|e| {
-                            let n: i32 = e.into();
-                            n
-                        })
-                        .collect(),
+                    paths.iter().map(ToString::to_string).collect(),
+                    data_types.into_iter().map(TSDataType::into).collect(),
+                    encodings.into_iter().map(TSEncoding::into).collect(),
                     compressors
-                        .iter()
-                        .map(|c| {
-                            let n: i32 = c.into();
-                            n
-                        })
+                        .into_iter()
+                        .map(TSCompressionType::into)
                         .collect(),
                     props_list,
                     attributes_list,
@@ -519,27 +526,22 @@ impl<'a> Session<'a> for RpcSession {
         }
     }
 
-    fn delete_timeseries(&mut self, paths: Vec<&str>) -> Result<(), Box<dyn Error>> {
+    fn delete_timeseries(&mut self, paths: Vec<&str>) -> Result<()> {
         if let Some(session_id) = self.session_id {
             let status = self
                 .client
-                .delete_timeseries(session_id, paths.iter().map(|x| x.to_string()).collect())?;
+                .delete_timeseries(session_id, paths.iter().map(ToString::to_string).collect())?;
             check_status(status)
         } else {
             fire_closed_error()
         }
     }
 
-    fn delete_data(
-        &mut self,
-        paths: Vec<&str>,
-        start_time: i64,
-        end_time: i64,
-    ) -> Result<(), Box<dyn Error>> {
+    fn delete_data(&mut self, paths: Vec<&str>, start_time: i64, end_time: i64) -> Result<()> {
         if let Some(session_id) = self.session_id {
             let status = self.client.delete_data(TSDeleteDataReq::new(
                 session_id,
-                paths.iter().map(|p| p.to_string()).collect(),
+                paths.into_iter().map(ToString::to_string).collect(),
                 start_time,
                 end_time,
             ))?;
@@ -556,7 +558,7 @@ impl<'a> Session<'a> for RpcSession {
         values: Vec<&str>,
         timestamp: i64,
         is_aligned: T,
-    ) -> Result<(), Box<dyn Error>>
+    ) -> Result<()>
     where
         T: Into<Option<bool>>,
     {
@@ -566,8 +568,8 @@ impl<'a> Session<'a> for RpcSession {
                 .insert_string_record(TSInsertStringRecordReq::new(
                     session_id,
                     device_id.to_string(),
-                    measurements.iter().map(|x| x.to_string()).collect(),
-                    values.iter().map(|x| x.to_string()).collect(),
+                    measurements.iter().map(ToString::to_string).collect(),
+                    values.iter().map(ToString::to_string).collect(),
                     timestamp,
                     is_aligned,
                 ))?;
@@ -577,7 +579,7 @@ impl<'a> Session<'a> for RpcSession {
         }
     }
 
-    fn get_time_zone(&mut self) -> Result<String, Box<dyn Error>> {
+    fn get_time_zone(&mut self) -> Result<String> {
         if let Some(session_id) = self.session_id {
             let resp = self.client.get_time_zone(session_id)?;
             let status = resp.status;
@@ -591,7 +593,7 @@ impl<'a> Session<'a> for RpcSession {
         }
     }
 
-    fn set_time_zone(&mut self, time_zone: &str) -> Result<(), Box<dyn Error>> {
+    fn set_time_zone(&mut self, time_zone: &str) -> Result<()> {
         if let Some(session_id) = self.session_id {
             let status = self
                 .client
@@ -606,7 +608,7 @@ impl<'a> Session<'a> for RpcSession {
         &'a mut self,
         statement: &str,
         timeout_ms: T,
-    ) -> Result<Box<dyn 'a + DataSet>, Box<dyn Error>>
+    ) -> Result<Box<dyn 'a + DataSet>>
     where
         T: Into<Option<i64>>,
     {
@@ -624,49 +626,49 @@ impl<'a> Session<'a> for RpcSession {
             let code = status.code;
             if code == SUCCESS_STATUS {
                 {
-                    let column_names: Vec<String> = resp.columns.unwrap();
-
-                    let column_name_index_map = match resp.column_name_index_map {
-                        Some(map) => map,
-                        None => {
-                            let mut map: BTreeMap<String, i32> = BTreeMap::new();
-                            for (index, name) in column_names.iter().enumerate() {
-                                map.insert(name.to_string(), index as i32);
+                    if let (Some(column_names), Some(data_type_list)) =
+                        (resp.columns, resp.data_type_list)
+                    {
+                        let column_name_index_map = match resp.column_name_index_map {
+                            Some(map) => map,
+                            None => {
+                                let mut map: BTreeMap<String, i32> = BTreeMap::new();
+                                for (index, name) in column_names.iter().enumerate() {
+                                    map.insert(name.to_string(), index as i32);
+                                }
+                                map
                             }
-                            map
+                        };
+
+                        let data_types: Vec<TSDataType> =
+                            data_type_list.iter().map(TSDataType::from).collect();
+
+                        let mut column_index_map: HashMap<usize, usize> = HashMap::new();
+
+                        let column_count = column_names.len();
+                        for (index, name) in column_names.iter().enumerate() {
+                            column_index_map
+                                .insert(*column_name_index_map.get(name).unwrap() as usize, index);
                         }
-                    };
 
-                    let data_types: Vec<TSDataType> = resp
-                        .data_type_list
-                        .unwrap()
-                        .iter()
-                        .map(|t| TSDataType::from(t))
-                        .collect();
-
-                    let mut column_index_map: HashMap<usize, usize> = HashMap::new();
-
-                    let column_count = column_names.len();
-                    for (index, name) in column_names.iter().enumerate() {
-                        column_index_map
-                            .insert(*column_name_index_map.get(name).unwrap() as usize, index);
+                        return Ok(Box::new(RpcDataSet {
+                            session: self,
+                            statement: statement.to_string(),
+                            query_id: resp.query_id.unwrap(),
+                            timestamp: -1,
+                            is_ignore_time_stamp: resp.ignore_time_stamp,
+                            query_data_set: resp.query_data_set.unwrap(),
+                            column_names: column_names,
+                            data_types: data_types,
+                            bitmaps: vec![0_u8; column_count],
+                            row_index: 0,
+                            column_index_map: column_index_map,
+                            column_name_index_map: column_name_index_map,
+                            closed: false,
+                        }));
+                    } else {
+                        Err("Can't get resources on execute_statement".into())
                     }
-
-                    return Ok(Box::new(RpcDataSet {
-                        session: self,
-                        statement: statement.to_string(),
-                        query_id: resp.query_id.unwrap(),
-                        timestamp: -1,
-                        is_ignore_time_stamp: resp.ignore_time_stamp,
-                        query_data_set: resp.query_data_set.unwrap(),
-                        column_names: column_names,
-                        data_types: data_types,
-                        bitmaps: vec![0_u8; column_count],
-                        row_index: 0,
-                        column_index_map: column_index_map,
-                        column_name_index_map: column_name_index_map,
-                        closed: false,
-                    }));
                 }
             } else {
                 if let Err(e) = check_status(status) {
@@ -684,7 +686,7 @@ impl<'a> Session<'a> for RpcSession {
         &'a mut self,
         statement: &str,
         timeout_ms: T,
-    ) -> Result<Box<dyn 'a + DataSet>, Box<dyn Error>>
+    ) -> Result<Box<dyn 'a + DataSet>>
     where
         T: Into<Option<i64>>,
     {
@@ -718,7 +720,7 @@ impl<'a> Session<'a> for RpcSession {
                     .data_type_list
                     .unwrap()
                     .iter()
-                    .map(|t| TSDataType::from(t))
+                    .map(TSDataType::from)
                     .collect();
 
                 let mut column_index_map: HashMap<usize, usize> = HashMap::new();
@@ -762,7 +764,7 @@ impl<'a> Session<'a> for RpcSession {
         values: Vec<Value>,
         timestamp: i64,
         is_aligned: T,
-    ) -> Result<(), Box<dyn Error>>
+    ) -> Result<()>
     where
         T: Into<Option<bool>>,
     {
@@ -775,7 +777,7 @@ impl<'a> Session<'a> for RpcSession {
             let status = self.client.insert_record(TSInsertRecordReq::new(
                 session_id,
                 device_id.to_string(),
-                measurements.iter().map(|x| x.to_string()).collect(),
+                measurements.iter().map(ToString::to_string).collect(),
                 values_bytes,
                 timestamp,
                 is_aligned,
@@ -793,7 +795,7 @@ impl<'a> Session<'a> for RpcSession {
         measurements: Vec<Vec<&str>>,
         values: Vec<Vec<super::Value>>,
         sorted: bool,
-    ) -> Result<(), Box<dyn Error>> {
+    ) -> Result<()> {
         let mut sorted_timestamps = timestamps;
         let mut sorted_measurements = measurements;
         let mut sorted_values = values;
@@ -824,7 +826,7 @@ impl<'a> Session<'a> for RpcSession {
                         device_id.to_string(),
                         sorted_measurements
                             .iter()
-                            .map(|vec| vec.iter().map(|s| s.to_string()).collect())
+                            .map(|vec| vec.iter().map(ToString::to_string).collect())
                             .collect(),
                         values_list,
                         sorted_timestamps,
@@ -838,11 +840,11 @@ impl<'a> Session<'a> for RpcSession {
 
     fn insert_records(
         &mut self,
-        device_ids: Vec<&str>,
+        prefix_path: Vec<&str>,
         measurements: Vec<Vec<&str>>,
         values: Vec<Vec<super::Value>>,
         timestamps: Vec<i64>,
-    ) -> Result<(), Box<dyn Error>> {
+    ) -> Result<()> {
         if let Some(session_id) = self.session_id {
             let values_list = values
                 .iter()
@@ -857,13 +859,10 @@ impl<'a> Session<'a> for RpcSession {
                 .collect();
             let status = self.client.insert_records(TSInsertRecordsReq {
                 session_id: session_id,
-                prefix_paths: device_ids
-                    .iter()
-                    .map(|device_id| device_id.to_string())
-                    .collect(),
+                prefix_paths: prefix_path.iter().map(ToString::to_string).collect(),
                 measurements_list: measurements
                     .iter()
-                    .map(|ms| ms.iter().map(|m| m.to_string()).collect())
+                    .map(|ms| ms.iter().map(ToString::to_string).collect())
                     .collect(),
                 values_list: values_list,
                 timestamps: timestamps,
@@ -875,9 +874,9 @@ impl<'a> Session<'a> for RpcSession {
         }
     }
 
-    fn insert_tablet(&mut self, tablet: &super::Tablet) -> Result<(), Box<dyn Error>> {
+    fn insert_tablet(&mut self, tablet: &super::Tablet) -> Result<()> {
         if let Some(session_id) = self.session_id {
-            let mut timestamps_list: Vec<u8> = Vec::new();
+            let mut timestamps_list: Vec<u8> = Vec::with_capacity(tablet.timestamps.len() * 8);
             tablet
                 .timestamps
                 .iter()
@@ -885,7 +884,7 @@ impl<'a> Session<'a> for RpcSession {
 
             let status = self.client.insert_tablet(TSInsertTabletReq {
                 session_id: session_id,
-                prefix_path: tablet.get_device_id(),
+                prefix_path: tablet.get_prefix_path(),
                 measurements: tablet
                     .measurement_schemas
                     .iter()
@@ -895,12 +894,8 @@ impl<'a> Session<'a> for RpcSession {
                 timestamps: timestamps_list,
                 types: tablet
                     .get_measurement_schemas()
-                    .iter()
-                    .map(|measurement_schema| {
-                        let t: i32;
-                        t = (&measurement_schema.data_type).into();
-                        t
-                    })
+                    .into_iter()
+                    .map(|measurement_schema| measurement_schema.data_type.into())
                     .collect(),
                 size: tablet.get_row_count() as i32,
                 is_aligned: Some(false),
@@ -911,11 +906,11 @@ impl<'a> Session<'a> for RpcSession {
         }
     }
 
-    fn insert_tablets(&mut self, tablets: Vec<&super::Tablet>) -> Result<(), Box<dyn Error>> {
+    fn insert_tablets(&mut self, tablets: Vec<&super::Tablet>) -> Result<()> {
         if let Some(session_id) = self.session_id {
             let status = self.client.insert_tablets(TSInsertTabletsReq {
                 session_id: session_id,
-                prefix_paths: tablets.iter().map(|t| t.get_device_id()).collect(),
+                prefix_paths: tablets.iter().map(|t| t.get_prefix_path()).collect(),
                 measurements_list: tablets
                     .iter()
                     .map(|tablet| {
@@ -926,13 +921,7 @@ impl<'a> Session<'a> for RpcSession {
                             .collect()
                     })
                     .collect(),
-                values_list: tablets
-                    .iter()
-                    .map(|tablet| {
-                        let values: Vec<u8> = (*tablet).into();
-                        values
-                    })
-                    .collect(),
+                values_list: tablets.iter().map(|tablet| Into::into(*tablet)).collect(),
                 timestamps_list: tablets
                     .iter()
                     .map(|tablet| {
@@ -949,9 +938,9 @@ impl<'a> Session<'a> for RpcSession {
                     .map(|tablet| {
                         tablet
                             .get_measurement_schemas()
-                            .iter()
+                            .into_iter()
                             .map(|f| {
-                                let t: i32 = (&(*f).data_type).into();
+                                let t: i32 = f.data_type.into();
                                 t
                             })
                             .collect()
@@ -969,13 +958,13 @@ impl<'a> Session<'a> for RpcSession {
         }
     }
 
-    fn execute_batch_statement(&mut self, statemens: Vec<&str>) -> Result<(), Box<dyn Error>> {
+    fn execute_batch_statement(&mut self, statemens: Vec<&str>) -> Result<()> {
         if let Some(session_id) = self.session_id {
             let status =
                 self.client
                     .execute_batch_statement(super::rpc::TSExecuteBatchStatementReq {
                         session_id: session_id,
-                        statements: statemens.iter().map(|f| f.to_string()).collect(),
+                        statements: statemens.iter().map(ToString::to_string).collect(),
                     })?;
             check_status(status)
         } else {
@@ -988,13 +977,13 @@ impl<'a> Session<'a> for RpcSession {
         paths: Vec<&str>,
         start_time: i64,
         end_time: i64,
-    ) -> Result<Box<dyn 'a + DataSet>, Box<dyn Error>> {
+    ) -> Result<Box<dyn 'a + DataSet>> {
         if let Some(session_id) = self.session_id {
             let resp = self
                 .client
                 .execute_raw_data_query(super::rpc::TSRawDataQueryReq {
                     session_id: session_id,
-                    paths: paths.iter().map(|f| f.to_string()).collect(),
+                    paths: paths.iter().map(ToString::to_string).collect(),
                     fetch_size: Some(self.config.fetch_size),
                     start_time: start_time,
                     end_time: end_time,
@@ -1019,10 +1008,8 @@ impl<'a> Session<'a> for RpcSession {
                         }
                     };
 
-                    let data_types: Vec<TSDataType> = data_types_list
-                        .iter()
-                        .map(|t| TSDataType::from(t))
-                        .collect();
+                    let data_types: Vec<TSDataType> =
+                        data_types_list.iter().map(TSDataType::from).collect();
 
                     let mut column_index_map: HashMap<usize, usize> = HashMap::new();
 
@@ -1064,7 +1051,7 @@ impl<'a> Session<'a> for RpcSession {
     fn execute_update_statement(
         &'a mut self,
         statement: &str,
-    ) -> Result<Option<Box<dyn 'a + DataSet>>, Box<dyn Error>> {
+    ) -> Result<Option<Box<dyn 'a + DataSet>>> {
         if let Some(session_id) = self.session_id {
             let resp = self
                 .client
@@ -1095,7 +1082,7 @@ impl<'a> Session<'a> for RpcSession {
                     };
 
                     let data_types: Vec<TSDataType> =
-                        data_type_list.iter().map(|t| TSDataType::from(t)).collect();
+                        data_type_list.iter().map(TSDataType::from).collect();
 
                     let mut column_index_map: HashMap<usize, usize> = HashMap::new();
 
@@ -1134,3 +1121,13 @@ impl<'a> Session<'a> for RpcSession {
         }
     }
 }
+
+impl Drop for RpcSession {
+    fn drop(&mut self) {
+        if let Some(session_id) = self.session_id {
+            self.close().unwrap_or_else(|err| {
+                eprint!("error closing the session {}, reason {}", session_id, err)
+            });
+        }
+    }
+}
